-- Agrega campo para registrar el cambio entregado en el cobro
ALTER TABLE tbl_movimientos_pagos
ADD cambio DECIMAL(15,2) AFTER importe_cobro;

-- Agrega a configuracion UNC para apertura de caja
ALTER TABLE cat_configuracion
ADD cashdrawer_unc TEXT AFTER desglosa_total;

UPDATE cat_configuracion
SET cashdrawer_unc = '';


-- Actualizacion de stored para aplicar cobros a pedidos
DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_cobroPedido`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_cobroPedido`(
	in_id_movimiento INT,
	in_cobro DECIMAL,
	in_cambio DECIMAL,
	in_descuento DECIMAL,
	in_forma_pago INT,
	in_factura BOOL,
	in_id_user INT,
	in_id_user_descuento INT
)
    SQL SECURITY INVOKER
    COMMENT 'Aplica el cobro a un pedido'
BEGIN
	UPDATE tbl_movimientos
	SET pagado = pagado + in_cobro - in_cambio + in_descuento,
		descuento = descuento + in_descuento,
		fecha_pago = NOW(),
		factura = in_factura,
		id_usuario_pago = in_id_user
	WHERE id_movimiento = in_id_movimiento;
	
	INSERT INTO tbl_movimientos_pagos
	SET id_movimiento = in_id_movimiento,
		importe_cobro = in_cobro,
		cambio = in_cambio,
		descuento = in_descuento,
		id_forma_pago = in_forma_pago,
		fecha_pago = NOW(),
		id_usuario_pago = in_id_user,
		id_usuario_descuento = in_id_user_descuento;
END$$

DELIMITER ;


-- Actualizacion de stored para cálculo del efectivo de la Caja
DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_efectivoEnCaja`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_efectivoEnCaja`()
    SQL SECURITY INVOKER
    COMMENT 'Obtiene el importe de efectivo en caja actualmente'
BEGIN
	DECLARE efectivo DECIMAL(15,2);
	DECLARE retiros DECIMAL(15,2);
	DECLARE pagos DECIMAL(15,2);
	DECLARE fecha_corte DATETIME;
	SET efectivo = 0;
	SET retiros = 0;
	SET pagos = 0;
	SET fecha_corte = '2000-01-01';
	
	-- Extrae efectivo real de caja
	SELECT cc.efectivo_real, cc.fecha_corte INTO efectivo, fecha_corte
	FROM tbl_cortes_caja AS cc
	ORDER BY cc.fecha_corte DESC
	LIMIT 1;
	
	-- Contabiliza los retiros realizados
	SELECT SUM(re.retiro) INTO retiros
	FROM tbl_retiros AS re
	WHERE re.fecha_retiro >= fecha_corte;
	
	-- Contabiliza los pagos recibidos
	SELECT SUM(p.importe_cobro - p.cambio) INTO pagos
	FROM tbl_movimientos_pagos AS p
	WHERE p.fecha_pago >= fecha_corte
		AND p.id_forma_pago = 0;
	
	-- Retorna el importe real de efectivo que debe haber en la caja
	SELECT efectivo + COALESCE(pagos, 0) - COALESCE(retiros, 0) AS efectivo,
		efectivo AS caja, COALESCE(pagos, 0) AS pagos, COALESCE(retiros, 0) AS retiros;
END$$

DELIMITER ;


-- Actualizacion de stored para carga de los parámetros de configuración
DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_configuracion`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_configuracion`()
    SQL SECURITY INVOKER
    COMMENT 'Obtiene la informacion de configuracion'
BEGIN
	SELECT cfg.id_sucursal,
		cfg.default_image,
		cfg.group_len,
		cfg.desglosa_total,
		cfg.cashdrawer_unc,
		cfg.url_existencias,
		cfg.url_sync_sucursales,
		cfg.url_sync_productos,
		cfg.url_sync_precios,
		cfg.url_sync_usuarios,
		cfg.url_sync_ventas
	FROM cat_configuracion AS cfg;
END$$

DELIMITER ;


-- Actualizacion de stored para actualización de parámetros de configuración
DELIMITER $$

DROP PROCEDURE IF EXISTS `stp_updateConfig`$$

CREATE DEFINER=`root`@`localhost` PROCEDURE `stp_updateConfig`(
	in_desglosa_total INT,
	in_cashdrawer_unc TEXT
)
    COMMENT 'Guarda las opciones de configuracion general'
BEGIN
	UPDATE cat_configuracion
	SET desglosa_total = in_desglosa_total,
		cashdrawer_unc = in_cashdrawer_unc;
END$$

DELIMITER ;